/*
 * @Description:
 * @Version: 2.0
 * @Autor: shimnx
 * @Date: 2021-08-04 09:07:37
 * @LastEditors: shimnx
 * @LastEditTime: 2022-07-26 15:23:40
 */
import { BrowserModule } from '@angular/platform-browser';
import { Injectable, NgModule } from '@angular/core';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
// import { MatDividerModule, MatToolbarModule, MatMenuModule, MatListModule } from '@angular/material';
import { AppRoutingModule } from '@/app/app-routing.module';
import { AppComponent } from '@/app/app.component';
import { PageNotFoundComponent } from '@/app/result/404/page-not-found.component';
import { NoPermissionComponent } from '@/app/result/401/no-permission.component';
import { HttpClientModule, HttpResponse } from '@angular/common/http';
import { DatePipe, registerLocaleData } from '@angular/common';
import { selfHttp } from './http-units.service';
import { SimpleReuseStrategy } from './SimpleReuseStrategy';
// mat 官方组件库
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatButtonModule } from '@angular/material/button';
import { MatInputModule } from '@angular/material/input';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatSidenavModule } from '@angular/material/sidenav';
import { ReactiveFormsModule, FormsModule } from '@angular/forms';
import { MatTableModule } from '@angular/material/table';
import { MatTabsModule } from '@angular/material/tabs';
import { apiList } from './app.api';
import { httpInterceptorProviders } from './http-intercepts/index'
import { MatSnackBar, MatSnackBarModule, MAT_SNACK_BAR_DEFAULT_OPTIONS } from '@angular/material/snack-bar';
import { SnackbarComponent } from './pages/snackbar/snackbar.component';
import { ngfModule } from 'angular-file';
import { MatSelectModule } from '@angular/material/select';
import { MtxGridModule } from '@ng-matero/extensions/data-grid';
import { MatDatepickerModule } from '@angular/material/datepicker';
import { MatNativeDateModule } from '@angular/material/core';
import { NzTableModule } from 'ng-zorro-antd/table';
import { MatChipsModule } from '@angular/material/chips';
import { HighchartsChartModule } from 'highcharts-angular';
import { HttpClient } from '@angular/common/http';
import { DemoButtonComponent } from './components/button/button.component';
import { MatMenuModule } from '@angular/material/menu';
import { MatTooltipModule } from '@angular/material/tooltip';

import {
    TranslateModule,
    TranslateLoader,
    TranslateService
} from '@ngx-translate/core';
import {
    HttpEvent,
    HttpInterceptor,
    HttpHandler,
    HttpRequest,
} from '@angular/common/http';
import { Observable } from 'rxjs';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { JwtModule } from '@auth0/angular-jwt';
// import { QuillModule } from 'ngx-Quill';
import { HTTP_INTERCEPTORS } from '@angular/common/http';
import { TokenComponent } from './token/token.component';
import { Router, RouteReuseStrategy } from '@angular/router';
import { tap } from 'rxjs/operators';
export function createTranslateLoader(http: HttpClient) {
    return new TranslateHttpLoader(http, './assets/i18n/', '.json');
}

import en from '@angular/common/locales/en';
registerLocaleData(en);
export class AddHeaderInterceptor implements HttpInterceptor {
    intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
        // Clone the request to add the new header
        const clonedRequest = req.clone({ headers: req.headers.set('Authorization', 'Bearer '+localStorage.getItem('token')) });

        // Pass the cloned request instead of the original request to the next handle
        return next.handle(clonedRequest);
    }
}
@Injectable()
export class NotificationInterceptor implements HttpInterceptor {
    constructor(private _snackBar: MatSnackBar) { }
    router: Router;
    intercept(req: HttpRequest<any>, next: HttpHandler) {
        // 对响应消息进行处理
        if (localStorage.getItem('token')) {
            let strings = localStorage.getItem('token').split("."); //截取token，获取载体
            var userinfo = JSON.parse(decodeURIComponent(window.atob(strings[1].replace(/-/g, "+").replace(/_/g, "/"))))
            var timestamp = new Date().getTime()

            if (timestamp > userinfo.exp * 1000) {
                localStorage.clear()
                location.reload()
            }

        }

        return next.handle(req).pipe(

            tap((event: HttpEvent<any>) => {

                if (
                    event instanceof HttpResponse &&
                    event.status >= 200 &&
                    event.status < 300
                ) {
                }
            },
                error => {
                    this._snackBar.openFromComponent(SnackbarComponent, {
                        data: { message: error.error.message || error.error.Message, action: 'error', },
                        verticalPosition: 'top',
                        panelClass: ['red-snackbar']
                    });

                    if (error.status == 403 || error.status == 401) {
                        window.location.href = 'https://kimihome.schaefflercn.com'


                    }
                    if (error.error.Message == "IDX10501: Signature validation failed. Unable to match key: \nkid: 'System.String'.\nExceptions caught:\n 'System.Text.StringBuilder'. \ntoken: 'System.IdentityModel.Tokens.Jwt.JwtSecurityToken'.") {

                        window.location.href = 'https://kimihome.schaefflercn.com'

                    }
                }
            )
        );
    }

}
@NgModule({
    declarations: [
        AppComponent,
        DemoButtonComponent,
        PageNotFoundComponent,
        NoPermissionComponent,
        SnackbarComponent,

        TokenComponent,
    ],
    imports: [
        // QuillModule.forRoot(),
        MatTooltipModule,
        MatMenuModule,
        MatTabsModule,
        HighchartsChartModule,
        MatChipsModule,
        ngfModule,
        MatDatepickerModule,
        MatNativeDateModule,

        MtxGridModule,
        MatButtonModule,
        MatSidenavModule,
        MatSelectModule,
        MatInputModule,
        MatFormFieldModule,
        MatIconModule,
        ReactiveFormsModule,
        FormsModule,
        HttpClientModule,
        BrowserModule,
        AppRoutingModule,
        BrowserAnimationsModule,
        MatCheckboxModule,
        NzTableModule,
        // MatDividerModule,
        // MatToolbarModule,
        // MatMenuModule,
        // MatListModule,
        MatTableModule,
        MatSnackBarModule,
        TranslateModule.forRoot({
            loader: {
                provide: TranslateLoader,
                useFactory: createTranslateLoader,
                deps: [HttpClient]
            }
        }),
        JwtModule.forRoot({
            config: {
                tokenGetter: () => {
                    return localStorage.getItem('token')
                },
            }
        })
    ],
    providers: [apiList, DatePipe, selfHttp,
        {
            provide: HTTP_INTERCEPTORS,
            useClass: AddHeaderInterceptor,
            multi: true,
        },
        {
            provide: HTTP_INTERCEPTORS,
            useClass: NotificationInterceptor,
            multi: true
        },
        { provide: RouteReuseStrategy, useClass: SimpleReuseStrategy },
        { provide: MAT_SNACK_BAR_DEFAULT_OPTIONS, useValue: { duration: 5000 } },
    ],
    entryComponents: [SnackbarComponent],
    bootstrap: [AppComponent]
})
export class AppModule { }
