/*
 * @Description:
 * @Version: 2.0
 * @Autor: shimnx
 * @Date: 2022-07-27 09:11:33
 * @LastEditors: shimnx
 * @LastEditTime: 2022-08-02 15:13:08
 */
import { TaskBoardConfig, ProjectModelConfig, TaskModel } from '@bryntum/taskboard';
import moment from 'moment';

export const projectConfig: Partial<ProjectModelConfig> = {};

class CustomTask extends TaskModel {
    static fields = [
        'priority',
        'progress'
    ];
}
const teamIcons = {
    medium  : 'pen-fancy',
    high : 'weight-hanging',
    low  : 'hammer',
};
export const taskBoardConfig: Partial<TaskBoardConfig> = {
    useDomTransition     : true,  // Experimental, transition moving cards using the editor
    stickyHeaders        : true,  // Want headers to stick to the top,
    showCollapseInHeader : false, // Hide column collapse tool
    showCountInHeader    : false, // Hide task count from column headers
    resourceImagePath    : 'assets/users/', // Url for resource avatar images
    css: {
        background: '#fff',
        bodyPadding: '.5em',

        columnBackground: '#f9f9f9',
        columnHeaderBorderTop: '5px solid #00893d',

        cardBorderTop: '5px solid #00893d',
        cardFooterFontSize: '.8em'
    },
    features: {
        columnHeaderMenu : { disabled : true },
        simpleTaskEdit   : { disabled : false },
        taskMenu: true,
        swimlaneDrag: true,
        taskEdit: {
            processItems(data) {
                (window as any).ScrumBoardComponent.component.openDrawer();
                console.log(data.taskRecord.stores)
                return false
            },
            items:{

                priority: {
                    type: 'combo',
                    items: Object.keys(teamIcons),
                    label: 'Priority',
                    onSelect:(event)=>{
                        console.log(event)
                    }
                }
            }

        },

    },
    swimlanes: [
        { id: 'GroupA', text: 'GroupA', color: '#FF9800' },
        { id: 'GroupB', text: 'GroupB', color: '#E62C27' },
        { id: 'GroupC', text: 'GroupC', color: '#1CAA52' }
    ],
    swimlaneField: 'group',
    // Add a button to show the context menu to card header
    headerItems: {
        menu: { type: 'taskMenu', order: 200 }
    },

    // Add a progress bar to card body
    bodyItems: {
        // Progress bar, 0 - 100
        progress: { type: 'progress', max: 100 },

        // Image
        image: { type: 'image', baseUrl: 'resources/' },

        // Separator line, uncomment to show
        // separator : { type : 'separator' },

        // Rating (starts), uncomment to show
        // rating   : { type : 'rating', max : 5 },

        // Team icon + text

        prio: {
            type: 'template',
            template: ({ value }) => {
                if(value=='medium'){
                    return  `<div class="medium">${value}</div>`
                }
                if(value=='high'){
                    return  `<div class="high">${value}</div>`
                }
                if(value=='low'){
                    return  `<div class="low">${value}</div>`
                }
            }
        },
        date: {
            type: 'template',
            template: ({ value }) => {
                const diff3 = moment(new Date()).diff(moment(value), 'days')
                if(diff3>0){
                    return `<div class='late'>${value}</div>`
                }else{
                    return `<div class='normal'>${value}</div>`
                }
            }
        }
    },

    // Add a template displaying priority to card footer
    footerItems: {
        assigner:{
            type: 'template',
            template: ({ value }) => {
                return  `<div>${value}</div>`
            }
        },
        dailyTask: { type: 'template', template: ({ value }) => `Daily Task ${value}` },
        action: { type: 'template', template: ({ value }) => `Action ${value}` },

    },

    project: {
        taskModelClass : CustomTask,
        // empty to start with
    },
    // tbar: [
    //     { type: 'taskfilterfield', value: '' }
    // ],
    columns: [
        { id: 'Todo', text: 'TO DO', color: 'deep-orange' },
        { id: 'Doing', text: 'DOING', color: 'orange' },
        { id: 'Done', text: 'DONE', color: '#00893d' }
    ],
    columnField: 'status'
};
