

'use strict';
(function (factory) {
	if (typeof module === 'object' && module.exports) {
		factory['default'] = factory;
		module.exports = factory;
	} else {
		factory(typeof Highcharts !== 'undefined' ? Highcharts : undefined);
	}
}(function (Highcharts) {

	let ZoomAndPan = {
		startPosition: false,
		container: null,
		start: e => {
			ZoomAndPan.startPosition = {
				x: e.clientX,
				y: e.clientY
			};
		},
		startZoom: touches => {
			ZoomAndPan.zoomReact =
				Math.abs(
					(touches[1].clientX - touches[0].clientX) *
					(touches[1].clientY - touches[0].clientY));

		},

		zooming: touches => {
			if (ZoomAndPan.zoomReact) {
				let newRect = Math.abs(
					(touches[1].clientX - touches[0].clientX) *
					(touches[1].clientY - touches[0].clientY));

				let scale = newRect / ZoomAndPan.zoomReact;

				if (Math.abs(scale - 1) > 0.1) {

					let zoom = ZoomAndPan.container.style.zoom ? parseFloat(ZoomAndPan.container.style.zoom) : 1;
					ZoomAndPan.container.style.zoom = zoom + (scale > 0 ? 0.1 : -0.1)

					ZoomAndPan.zoomReact = newRect;
				}
			}
		},
		moving: e => {
			if (!ZoomAndPan.startPosition) {
				return false;
			}
			let change = {
				x: e.clientX - ZoomAndPan.startPosition.x,
				y: e.clientY - ZoomAndPan.startPosition.y
			};

			ZoomAndPan.container.style.left = (ZoomAndPan.container.style.left ? parseInt(ZoomAndPan.container.style.left) : 0) + change.x + 'px';
			ZoomAndPan.container.style.top = (ZoomAndPan.container.style.top ? parseInt(ZoomAndPan.container.style.top) : 0) + change.y + 'px';

			ZoomAndPan.startPosition = {
				x: e.clientX,
				y: e.clientY
			};
		},
		end: () => {
			ZoomAndPan.startPosition = null;
			ZoomAndPan.zoomReact = null;
		},

		zoom: isZoomIn => {



			console.log(ZoomAndPan.container)
			let zoom = ZoomAndPan.container.style.zoom ? parseFloat(ZoomAndPan.container.style.zoom) : 1;
			ZoomAndPan.container.style.zoom = zoom + (isZoomIn ? -0.1 : 0.1);
		},
		nodefault: e => {
			e.cancelBubble = true;
			e.preventDefault();
			e.stopPropagation();
			e.stopImmediatePropagation();
			return false;
		}
	};






	Highcharts.wrap(Highcharts.Chart.prototype, 'init', function (proceed) {


		let userOptions = arguments[1];
		let series = userOptions.series && userOptions.series[0];

		if (series && series.type === 'organization' && series.zoom) {


			// 添加 DOM 及事件
			let container = typeof this.renderTo === 'string' ? document.getElementById(this.renderTo) : this.renderTo;
			if (container) {

				let wrapper = document.createElement('div');
				wrapper.className = 'highcharts-wrapper';
				wrapper.style.width = '100%';
				wrapper.style.height = '100%';
				wrapper.style.position = 'relative';
				container.appendChild(wrapper);
				this.renderTo = wrapper;

				Highcharts.addEvent(wrapper, 'mousedown', e => {
					ZoomAndPan.start(e);
				});

				Highcharts.addEvent(wrapper, 'touchstart', e => {
					if (e.touches.length > 1) {
						ZoomAndPan.startZoom(e.touches)
					} else {
						ZoomAndPan.start(e.touches[0]);
					}
				});

				Highcharts.addEvent(wrapper, 'mousemove', e => {
					ZoomAndPan.moving(e);
				});

				Highcharts.addEvent(wrapper, 'touchmove', e => {
					if (e.touches.length > 1) {
						ZoomAndPan.zooming(e.touches)
					} else {
						ZoomAndPan.moving(e.touches[0])
					}
				});

				Highcharts.addEvent(document.body, 'mouseup', e => {
					ZoomAndPan.end();
				});

				Highcharts.addEvent(document.body, 'touchend', e => {
					ZoomAndPan.end();
				});

				Highcharts.addEvent(wrapper, 'mousewheel', e => {
					if (e.ctrlKey) {
						let zoomIn = e.deltaY > 0;
						ZoomAndPan.zoom(zoomIn);
					}
					return ZoomAndPan.nodefault(e);
				});

				// Highcharts.addEvent(this, 'redraw', function() {
				// 	console.log(this.container);
				// })


				// 处理数据
				let RootId = '00000000-0000-0000-0000-000000000000',
					MaxColumnLength = 3,
					RootNode = null,
					SecondLevelNode = [],
					links = [],
					nodes = {};


				series.nodes.forEach(n => {
					nodes[n.id] = n;
					if (n.parentId === RootId) {
						RootNode = n;
					}
				});


				series.nodes.forEach(n => {
					if (n.parentId === RootNode.id) {
						n.children = [];
						n.layout = 'hanging';
						SecondLevelNode.push(n);
					}
				});

				series.nodes.forEach(n => {
					if (n.parentId !== RootId && n.parentId !== RootNode.id) {
						if (!nodes[n.parentId].children) {
							return;
						}
						nodes[n.parentId].children.push(n);
					}
				});

				SecondLevelNode.forEach(n => {
					let parent = n;
					n.children.forEach((child, i) => {
						if (i > 0 && i % MaxColumnLength === 0) {
							let newNode = {
								id: Highcharts.uniqueKey(),
								virtrual: true,
								_target: n.id,
								layout: 'hanging',
								dataLabels: {
									format: ''
								}
							};

							links.push([
								RootNode.id,
								newNode.id
							]);
							parent = newNode;
							series.nodes.push(newNode);
						}
						links.push([parent.id, child.id]);
					});

					links.push(
						[RootNode.id, n.id]
					);
				});


				series.data = links;

			}
		}

		proceed.apply(this, Array.prototype.slice.call(arguments, 1));
	});


	// 包裹 translateNode 函数，针对 virtrual 添加 display = 'none' shapeArgs
	// Highcharts.wrap(Highcharts.seriesTypes.organization.prototype, 'translateNode', function (proceed) {
	// 	let node = arguments[1];

	// 	console.log(node.options.height, this.options.nodeHeight);
	// 	proceed.apply(this, Array.prototype.slice.call(arguments, 1));

	// 	// hide virtrual node
	// 	// let node = arguments[1];
	// 	console.log(node.shapeArgs);
	// 	if (node.options.virtrual) {
	// 		node.shapeArgs.display = 'none';
	// 	}
	// });


	Highcharts.seriesTypes.organization.prototype.translateNode = function (node, column) {
		Highcharts.seriesTypes.organization.prototype._translateNode.call(this, node, column);
		var parentNode = node.hangsFrom, indent = this.options.hangingIndent || 0, sign = this.chart.inverted ? -1 : 1, shapeArgs = node.shapeArgs, indentLogic = this.options.hangingIndentTranslation, minLength = this.options.minNodeLength || 10;
		if (parentNode) {
			if (indentLogic === 'cumulative') {
				// Move to the right:
				shapeArgs.height -= indent;
				shapeArgs.y -= sign * indent;
				while (parentNode) {
					shapeArgs.y += sign * indent;
					parentNode = parentNode.hangsFrom;
				}
			}
			else if (indentLogic === 'shrink') {
				// Resize the node:
				while (parentNode &&
					shapeArgs.height > indent + minLength) {
					shapeArgs.height -= indent;
					parentNode = parentNode.hangsFrom;
				}
			}
			else {
				// indentLogic === "inherit"
				// Do nothing (v9.3.2 and prev versions):
				shapeArgs.height -= indent;
				if (!this.chart.inverted) {
					shapeArgs.y += indent;
				}
			}
		}
		node.nodeHeight = this.chart.inverted ?
			shapeArgs.width :
			shapeArgs.height;
	}

	Highcharts.seriesTypes.organization.prototype._translateNode = function (node, column) {
		var translationFactor = this.translationFactor,
			chart = this.chart, options = this.options, sum = node.getSum(),
			nodeHeight = options.nodeHeight || Math.max(Math.round(sum * translationFactor), this.options.minLinkWidth),
			nodeWidth = Math.round(this.nodeWidth),
			crisp = Math.round(options.borderWidth) % 2 / 2,
			nodeOffset = column.offset(node, translationFactor),
			fromNodeTop = Math.floor(Highcharts.pick(nodeOffset.absoluteTop, (column.top(translationFactor) +
				nodeOffset.relativeTop))) + crisp, left = Math.floor(this.colDistance * node.column +
					options.borderWidth / 2) + Highcharts.relativeLength(node.options.offsetHorizontal || 0, nodeWidth) +
					crisp, nodeLeft = chart.inverted ?
						chart.plotSizeX - left :
						left;
		node.sum = sum;
		// If node sum is 0, don't render the rect #12453
		if (sum) {
			// Draw the node
			node.shapeType = 'rect';
			node.nodeX = nodeLeft;
			node.nodeY = fromNodeTop;
			var x = nodeLeft, y = fromNodeTop, width = node.options.width || options.width || nodeWidth, height = node.options.height || options.height || nodeHeight;
			if (chart.inverted) {
				x = nodeLeft - nodeWidth;
				y = chart.plotSizeY - fromNodeTop - nodeHeight;
				width = node.options.height || options.height || nodeWidth;
				height = node.options.width || options.width || nodeHeight;
			}
			// Calculate data label options for the point
			node.dlOptions = Highcharts.seriesTypes.organization.getDLOptions({
				level: this.mapOptionsToLevel[node.level],
				optionsPoint: node.options
			});
			// Pass test in drawPoints
			node.plotX = 1;
			node.plotY = 1;
			// Set the anchor position for tooltips
			node.tooltipPos = chart.inverted ? [
				chart.plotSizeY - y - height / 2,
				chart.plotSizeX - x - width / 2
			] : [
				x + width / 2,
				y + height / 2
			];
			node.shapeArgs = {
				x: x,
				y: y,
				width: width,
				height: height,
				display: node.hasShape() ? '' : 'none'
			};
		}
		else {
			node.dlOptions = {
				enabled: false
			};
		}

		if (node.options.virtrual) {
			node.shapeArgs.display = 'none';
		}
	}


	// 重写 translateLink 函数，针对不同的情况略微调整生成 Link 路径
	Highcharts.seriesTypes.organization.prototype.translateLink = function (point) {
		var fromNode = point.fromNode, toNode = point.toNode,
			xOffset = 0,
			yOffset = 0,
			virtrual = false,
			path = [];


		if (toNode.options.virtrual) {
			return false;
		} else if (fromNode.options.virtrual) {
			virtrual = true;
			if (toNode.column === 2) {
				fromNode = this.nodeLookup[fromNode.options._target];
				yOffset = true;
			} else {
				xOffset = true;
			}
		}



		var crisp = Math.round(this.options.linkLineWidth) % 2 / 2,
			x1 = Math.floor(fromNode.shapeArgs.x + fromNode.shapeArgs.width) + crisp,
			y1 = Math.floor(fromNode.shapeArgs.y + fromNode.shapeArgs.height / 2) + crisp,
			x2 = Math.floor(toNode.shapeArgs.x) + crisp,
			y2 = Math.floor(toNode.shapeArgs.y + toNode.shapeArgs.height / 2) + crisp,
			xMiddle,
			hangingIndent = this.options.hangingIndent,
			toOffset = toNode.options.offset,
			inverted = this.chart.inverted;

		if (xOffset) {
			xOffset = -(this.colDistance - this.nodeWidth) / 2 - this.options.linkRadius;
		}


		if (inverted) {
			x1 -= fromNode.shapeArgs.width;
			x2 += toNode.shapeArgs.width;
		}

		if (virtrual) {
			xMiddle = x2 = Math.floor(toNode.shapeArgs.x + toNode.shapeArgs.width / 2) + crisp;
		} else {
			xMiddle = Math.floor(x2 + (inverted ? 1 : -1) * (this.colDistance - this.nodeWidth) / 2) + crisp;
		}


		if (toNode.hangsFrom === fromNode) {
			if (this.chart.inverted) {
				y1 = Math.floor(fromNode.shapeArgs.y + fromNode.shapeArgs.height - hangingIndent / 2) + crisp;
				y2 = (toNode.shapeArgs.y +
					toNode.shapeArgs.height);
			}
			else {
				y1 = Math.floor(fromNode.shapeArgs.y +
					hangingIndent / 2) + crisp;
			}
			xMiddle = x2 = Math.floor(toNode.shapeArgs.x + toNode.shapeArgs.width / 2) + crisp;
		}

		if (yOffset) {
			xMiddle += (this.colDistance - this.nodeWidth) + hangingIndent / 2;
			let offset = fromNode.shapeArgs.height / 2 - hangingIndent / 2;
			y1 += offset;
			y2 += offset + hangingIndent / 2;
		}

		path = [
			['M', x1 + xOffset, y1],
			['L', xMiddle, y1],
			['L', xMiddle, y2],
			['L', x2, y2]
		];

		if (yOffset) {
			path.push(['L', x2, y2 - hangingIndent / 2])
		}


		point.plotY = 1;
		point.shapeType = 'path';
		point.shapeArgs = {
			d: Highcharts.seriesTypes.organization.curvedPath(path, this.options.linkRadius)
		};
	}





	Highcharts.Chart.prototype.callbacks.push(function (chart) {
		console.log('sd,sdsd');

		let series = chart.series && chart.series.length && chart.series[0];
		if (series.type === 'organization' && series.userOptions.zoom === true) {
			let nodeHeight = series.userOptions.nodeWidth || 80,
				nodeWidth = series.userOptions.nodeHeight || 100,
				nodePadding = series.userOptions.nodePadding || 10,
				targetHeight = (series.nodeColumns.length * 2 - 1) * nodeHeight,
				maxColumnLength = 0,
				targetWidth = null,
				columnLength = series.nodeColumns.length - 1;

			// // calc max node length in each column and link's order
			for (let i = columnLength; i >= 0; i--) {
				let column = series.nodeColumns[i];

				if (column.length > maxColumnLength) {
					maxColumnLength = column.length;
				}
			}

			// update size
			targetWidth = maxColumnLength * nodeWidth + (maxColumnLength + 1) * nodePadding;
			chart.update({
				chart: {
					width: targetWidth,
					height: targetHeight
				}
			});


			if (!chart.options.chart.forExport) {
				ZoomAndPan.container = chart.container
			}
			chart.container.style.left = -(chart.chartWidth - chart.renderTo.clientWidth) / 2 + 'px';
			// chart.container.style.top = -(chart.chartHeight / 2 - chart.renderTo.clientHeight / 2) + 'px'

			// align center
			// setTimeout(() => {
			//    chart.renderTo.scrollLeft = (chart.renderTo.scrollWidth - chart.renderTo.clientWidth) / 2
			// }, 500)

		}
	});
}));